@@ -71,6 +71,12 @@ static snd_pcm_sframes_t (*writei_func)(snd_pcm_t *handle, const void *buffer, s
 static snd_pcm_sframes_t (*readn_func)(snd_pcm_t *handle, void **bufs, snd_pcm_uframes_t size);
 static snd_pcm_sframes_t (*writen_func)(snd_pcm_t *handle, void **bufs, snd_pcm_uframes_t size);
 
+enum {
+	VUMETER_NONE,
+	VUMETER_MONO,
+	VUMETER_STEREO
+};
+
 static char *command;
 static snd_pcm_t *handle;
 static struct {
@@ -81,7 +87,6 @@ static struct {
 static int timelimit = 0;
 static int quiet_mode = 0;
 static int file_type = FORMAT_DEFAULT;
-static unsigned int sleep_min = 0;
 static int open_mode = 0;
 static snd_pcm_stream_t stream = SND_PCM_STREAM_PLAYBACK;
 static int mmap_flag = 0;
@@ -97,9 +102,11 @@ static int avail_min = -1;
 static int start_delay = 0;
 static int stop_delay = 0;
 static int verbose = 0;
+static int vumeter = VUMETER_NONE;
 static int buffer_pos = 0;
 static size_t bits_per_sample, bits_per_frame;
 static size_t chunk_bytes;
+static int test_position = 0;
 static snd_output_t *log;
 
 static int fd = -1;
@@ -164,7 +171,6 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "-f, --format=FORMAT     sample format (case insensitive)\n"
 "-r, --rate=#            sample rate\n"
 "-d, --duration=#        interrupt after # seconds\n"
-"-s, --sleep-min=#       min ticks to sleep\n"
 "-M, --mmap              mmap stream\n"
 "-N, --nonblock          nonblocking mode\n"
 "-F, --period-time=#     distance between interrupts is # microseconds\n"
@@ -176,7 +182,13 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "                        (relative to buffer size if <= 0)\n"
 "-T, --stop-delay=#      delay for automatic PCM stop is # microseconds from xrun\n"
 "-v, --verbose           show PCM structure and setup (accumulative)\n"
-"-I, --separate-channels one file for each channel\n")
+"-V, --vumeter=TYPE      enable VU meter (TYPE: mono or stereo)\n"
+"-I, --separate-channels one file for each channel\n"
+"    --disable-resample  disable automatic rate resample\n"
+"    --disable-channels  disable automatic channel conversions\n"
+"    --disable-format    disable automatic format conversions\n"
+"    --disable-softvol   disable software volume control (softvol)\n"
+"    --test-position     test ring buffer position\n")
 		, command);
 	printf(_("Recognized sample formats are:"));
 	for (k = 0; k < SND_PCM_FORMAT_LAST; ++k) {
@@ -333,13 +345,18 @@ static void signal_handler(int sig)
 enum {
 	OPT_VERSION = 1,
 	OPT_PERIOD_SIZE,
-	OPT_BUFFER_SIZE
+	OPT_BUFFER_SIZE,
+	OPT_DISABLE_RESAMPLE,
+	OPT_DISABLE_CHANNELS,
+	OPT_DISABLE_FORMAT,
+	OPT_DISABLE_SOFTVOL,
+	OPT_TEST_POSITION
 };
 
 int main(int argc, char *argv[])
 {
 	int option_index;
-	char *short_options = "hnlLD:qt:c:f:r:d:s:MNF:A:R:T:B:vIPC";
+	char *short_options = "hnlLD:qt:c:f:r:d:MNF:A:R:T:B:vV:IPC";
 	static struct option long_options[] = {
 		{"help", 0, 0, 'h'},
 		{"version", 0, 0, OPT_VERSION},
@@ -353,7 +370,6 @@ int main(int argc, char *argv[])
 		{"format", 1, 0, 'f'},
 		{"rate", 1, 0, 'r'},
 		{"duration", 1, 0 ,'d'},
-		{"sleep-min", 1, 0, 's'},
 		{"mmap", 0, 0, 'M'},
 		{"nonblock", 0, 0, 'N'},
 		{"period-time", 1, 0, 'F'},
@@ -364,9 +380,15 @@ int main(int argc, char *argv[])
 		{"buffer-time", 1, 0, 'B'},
 		{"buffer-size", 1, 0, OPT_BUFFER_SIZE},
 		{"verbose", 0, 0, 'v'},
+		{"vumeter", 1, 0, 'V'},
 		{"separate-channels", 0, 0, 'I'},
 		{"playback", 0, 0, 'P'},
 		{"capture", 0, 0, 'C'},
+		{"disable-resample", 0, 0, OPT_DISABLE_RESAMPLE},
+		{"disable-channels", 0, 0, OPT_DISABLE_CHANNELS},
+		{"disable-format", 0, 0, OPT_DISABLE_FORMAT},
+		{"disable-softvol", 0, 0, OPT_DISABLE_SOFTVOL},
+		{"test-position", 0, 0, OPT_TEST_POSITION},
 		{0, 0, 0, 0}
 	};
 	char *pcm_name = "default";
@@ -478,9 +500,6 @@ int main(int argc, char *argv[])
 		case 'd':
 			timelimit = strtol(optarg, NULL, 0);
 			break;
-		case 's':
-			sleep_min = strtol(optarg, NULL, 0);
-			break;
 		case 'N':
 			nonblock = 1;
 			open_mode |= SND_PCM_NONBLOCK;
@@ -508,6 +527,16 @@ int main(int argc, char *argv[])
 			break;
 		case 'v':
 			verbose++;
+			if (verbose > 1 && !vumeter)
+				vumeter = VUMETER_MONO;
+			break;
+		case 'V':
+			if (*optarg == 's')
+				vumeter = VUMETER_STEREO;
+			else if (*optarg == 'm')
+				vumeter = VUMETER_MONO;
+			else
+				vumeter = VUMETER_NONE;
 			break;
 		case 'M':
 			mmap_flag = 1;
@@ -526,6 +555,21 @@ int main(int argc, char *argv[])
 			if (file_type == FORMAT_DEFAULT)
 				file_type = FORMAT_WAVE;
 			break;
+		case OPT_DISABLE_RESAMPLE:
+			open_mode |= SND_PCM_NO_AUTO_RESAMPLE;
+			break;
+		case OPT_DISABLE_CHANNELS:
+			open_mode |= SND_PCM_NO_AUTO_CHANNELS;
+			break;
+		case OPT_DISABLE_FORMAT:
+			open_mode |= SND_PCM_NO_AUTO_FORMAT;
+			break;
+		case OPT_DISABLE_SOFTVOL:
+			open_mode |= SND_PCM_NO_SOFTVOL;
+			break;
+		case OPT_TEST_POSITION:
+			test_position = 1;
+			break;
 		default:
 			fprintf(stderr, _("Try `%s --help' for more information.\n"), command);
 			return 1;
@@ -727,15 +771,29 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 	check_wavefile_space(buffer, len, blimit);
 	test_wavefile_read(fd, buffer, &size, len, __LINE__);
 	f = (WaveFmtBody*) buffer;
-	if (LE_SHORT(f->format) != WAV_PCM_CODE) {
-		error(_("can't play not PCM-coded WAVE-files"));
+	if (LE_SHORT(f->format) == WAV_FMT_EXTENSIBLE) {
+		WaveFmtExtensibleBody *fe = (WaveFmtExtensibleBody*)buffer;
+		if (len < sizeof(WaveFmtExtensibleBody)) {
+			error(_("unknown length of extensible 'fmt ' chunk (read %u, should be %u at least)"),
+					len, (u_int)sizeof(WaveFmtExtensibleBody));
+			exit(EXIT_FAILURE);
+		}
+		if (memcmp(fe->guid_tag, WAV_GUID_TAG, 14) != 0) {
+			error(_("wrong format tag in extensible 'fmt ' chunk"));
+			exit(EXIT_FAILURE);
+		}
+		f->format = fe->guid_format;
+	}
+        if (LE_SHORT(f->format) != WAV_FMT_PCM &&
+            LE_SHORT(f->format) != WAV_FMT_IEEE_FLOAT) {
+                error(_("can't play WAVE-file format 0x%04x which is not PCM or FLOAT encoded"), LE_SHORT(f->format));
 		exit(EXIT_FAILURE);
 	}
-	if (LE_SHORT(f->modus) < 1) {
-		error(_("can't play WAVE-files with %d tracks"), LE_SHORT(f->modus));
+	if (LE_SHORT(f->channels) < 1) {
+		error(_("can't play WAVE-files with %d tracks"), LE_SHORT(f->channels));
 		exit(EXIT_FAILURE);
 	}
-	hwparams.channels = LE_SHORT(f->modus);
+	hwparams.channels = LE_SHORT(f->channels);
 	switch (LE_SHORT(f->bit_p_spl)) {
 	case 8:
 		if (hwparams.format != DEFAULT_FORMAT &&
@@ -770,7 +828,10 @@ static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 		}
 		break;
 	case 32:
-		hwparams.format = SND_PCM_FORMAT_S32_LE;
+                if (LE_SHORT(f->format) == WAV_FMT_PCM)
+                        hwparams.format = SND_PCM_FORMAT_S32_LE;
+                else if (LE_SHORT(f->format) == WAV_FMT_IEEE_FLOAT)
+                        hwparams.format = SND_PCM_FORMAT_FLOAT_LE;
 		break;
 	default:
 		error(_(" can't play WAVE-files with sample %d bits wide"),
@@ -869,7 +930,6 @@ static void set_params(void)
 	snd_pcm_uframes_t buffer_size;
 	int err;
 	size_t n;
-	snd_pcm_uframes_t xfer_align;
 	unsigned int rate;
 	snd_pcm_uframes_t start_threshold, stop_threshold;
 	snd_pcm_hw_params_alloca(&params);
@@ -971,16 +1031,6 @@ static void set_params(void)
 		exit(EXIT_FAILURE);
 	}
 	snd_pcm_sw_params_current(handle, swparams);
-	err = snd_pcm_sw_params_get_xfer_align(swparams, &xfer_align);
-	if (err < 0) {
-		error(_("Unable to obtain xfer align\n"));
-		exit(EXIT_FAILURE);
-	}
-	if (sleep_min)
-		xfer_align = 1;
-	err = snd_pcm_sw_params_set_sleep_min(handle, swparams,
-					      sleep_min);
-	assert(err >= 0);
 	if (avail_min < 0)
 		n = chunk_size;
 	else
@@ -988,7 +1038,7 @@ static void set_params(void)
 	err = snd_pcm_sw_params_set_avail_min(handle, swparams, n);
 
 	/* round up to closest transfer boundary */
-	n = (buffer_size / xfer_align) * xfer_align;
+	n = buffer_size;
 	if (start_delay <= 0) {
 		start_threshold = n + (double) rate * start_delay / 1000000;
 	} else
@@ -1006,9 +1056,6 @@ static void set_params(void)
 	err = snd_pcm_sw_params_set_stop_threshold(handle, swparams, stop_threshold);
 	assert(err >= 0);
 
-	err = snd_pcm_sw_params_set_xfer_align(handle, swparams, xfer_align);
-	assert(err >= 0);
-
 	if (snd_pcm_sw_params(handle, swparams) < 0) {
 		error(_("unable to install sw params:"));
 		snd_pcm_sw_params_dump(swparams, log);
@@ -1027,6 +1074,30 @@ static void set_params(void)
 		exit(EXIT_FAILURE);
 	}
 	// fprintf(stderr, "real chunk_size = %i, frags = %i, total = %i\n", chunk_size, setup.buf.block.frags, setup.buf.block.frags * chunk_size);
+
+	/* stereo VU-meter isn't always available... */
+	if (vumeter == VUMETER_STEREO) {
+		if (hwparams.channels != 2 || !interleaved || verbose > 2)
+			vumeter = VUMETER_MONO;
+	}
+
+	/* show mmap buffer arragment */
+	if (mmap_flag && verbose) {
+		const snd_pcm_channel_area_t *areas;
+		snd_pcm_uframes_t offset;
+		int i;
+		err = snd_pcm_mmap_begin(handle, &areas, &offset, &chunk_size);
+		if (err < 0) {
+			error("snd_pcm_mmap_begin problem: %s", snd_strerror(err));
+			exit(EXIT_FAILURE);
+		}
+		for (i = 0; i < hwparams.channels; i++)
+			fprintf(stderr, "mmap_area[%i] = %p,%u,%u (%u)\n", i, areas[i].addr, areas[i].first, areas[i].step, snd_pcm_format_physical_width(hwparams.format));
+		/* not required, but for sure */
+		snd_pcm_mmap_commit(handle, offset, 0);
+	}
+
+	buffer_frames = buffer_size;	/* for position test */
 }
 
 #ifndef timersub
@@ -1112,23 +1183,101 @@ static void suspend(void)
 		fprintf(stderr, _("Done.\n"));
 }
 
+static void print_vu_meter_mono(int perc, int maxperc)
+{
+	const int bar_length = 50;
+	char line[80];
+	int val;
+
+	for (val = 0; val <= perc * bar_length / 100 && val < bar_length; val++)
+		line[val] = '#';
+	for (; val <= maxperc * bar_length / 100 && val < bar_length; val++)
+		line[val] = ' ';
+	line[val] = '+';
+	for (++val; val <= bar_length; val++)
+		line[val] = ' ';
+	if (maxperc > 99)
+		sprintf(line + val, "| MAX");
+	else
+		sprintf(line + val, "| %02i%%", maxperc);
+	fputs(line, stdout);
+	if (perc > 100)
+		printf(_(" !clip  "));
+}
+
+static void print_vu_meter_stereo(int *perc, int *maxperc)
+{
+	const int bar_length = 35;
+	char line[80];
+	int c;
+
+	memset(line, ' ', sizeof(line) - 1);
+	line[bar_length + 3] = '|';
+
+	for (c = 0; c < 2; c++) {
+		int p = perc[c] * bar_length / 100;
+		char tmp[4];
+		if (p > bar_length)
+			p = bar_length;
+		if (c)
+			memset(line + bar_length + 6 + 1, '#', p);
+		else
+			memset(line + bar_length - p - 1, '#', p);
+		p = maxperc[c] * bar_length / 100;
+		if (p > bar_length)
+			p = bar_length;
+		if (c)
+			line[bar_length + 6 + 1 + p] = '+';
+		else
+			line[bar_length - p - 1] = '+';
+		if (maxperc[c] > 99)
+			sprintf(tmp, "MAX");
+		else
+			sprintf(tmp, "%02d%%", maxperc[c]);
+		if (c)
+			memcpy(line + bar_length + 3 + 1, tmp, 3);
+		else
+			memcpy(line + bar_length, tmp, 3);
+	}
+	line[bar_length * 2 + 6 + 2] = 0;
+	fputs(line, stdout);
+}
+
+static void print_vu_meter(signed int *perc, signed int *maxperc)
+{
+	if (vumeter == VUMETER_STEREO)
+		print_vu_meter_stereo(perc, maxperc);
+	else
+		print_vu_meter_mono(*perc, *maxperc);
+}
+
 /* peak handler */
 static void compute_max_peak(u_char *data, size_t count)
 {
-	signed int val, max, max_peak = 0, perc;
+	signed int val, max, perc[2], max_peak[2];
 	static	int	run = 0;
 	size_t ocount = count;
 	int	format_little_endian = snd_pcm_format_little_endian(hwparams.format);	
+	int ichans, c;
 
+	if (vumeter == VUMETER_STEREO)
+		ichans = 2;
+	else
+		ichans = 1;
+
+	memset(max_peak, 0, sizeof(max_peak));
 	switch (bits_per_sample) {
 	case 8: {
 		signed char *valp = (signed char *)data;
 		signed char mask = snd_pcm_format_silence(hwparams.format);
+		c = 0;
 		while (count-- > 0) {
 			val = *valp++ ^ mask;
 			val = abs(val);
-			if (max_peak < val)
-				max_peak = val;
+			if (max_peak[c] < val)
+				max_peak[c] = val;
+			if (vumeter == VUMETER_STEREO)
+				c = !c;
 		}
 		break;
 	}
@@ -1138,14 +1287,18 @@ static void compute_max_peak(u_char *data, size_t count)
 		signed short sval;
 
 		count /= 2;
+		c = 0;
 		while (count-- > 0) {
 			if (format_little_endian)
 				sval = __le16_to_cpu(*valp);
-			else	sval = __be16_to_cpu(*valp);
+			else
+				sval = __be16_to_cpu(*valp);
 			sval = abs(sval) ^ mask;
-			if (max_peak < sval)
-				max_peak = sval;
+			if (max_peak[c] < sval)
+				max_peak[c] = sval;
 			valp++;
+			if (vumeter == VUMETER_STEREO)
+				c = !c;
 		}
 		break;
 	}
@@ -1154,6 +1307,7 @@ static void compute_max_peak(u_char *data, size_t count)
 		signed int mask = snd_pcm_format_silence_32(hwparams.format);
 
 		count /= 3;
+		c = 0;
 		while (count-- > 0) {
 			if (format_little_endian) {
 				val = valp[0] | (valp[1]<<8) | (valp[2]<<16);
@@ -1165,30 +1319,37 @@ static void compute_max_peak(u_char *data, size_t count)
 				val |= 0xff<<24;	/* Negate upper bits too */
 			}
 			val = abs(val) ^ mask;
-			if (max_peak < val)
-				max_peak = val;
+			if (max_peak[c] < val)
+				max_peak[c] = val;
 			valp += 3;
+			if (vumeter == VUMETER_STEREO)
+				c = !c;
 		}
 		break;
 	}
 	case 32: {
 		signed int *valp = (signed int *)data;
 		signed int mask = snd_pcm_format_silence_32(hwparams.format);
+
 		count /= 4;
+		c = 0;
 		while (count-- > 0) {
 			if (format_little_endian)
 				val = __le32_to_cpu(*valp);
-			else	val = __be32_to_cpu(*valp);
+			else
+				val = __be32_to_cpu(*valp);
 			val = abs(val) ^ mask;
-			if (max_peak < val)
-				max_peak = val;
+			if (max_peak[c] < val)
+				max_peak[c] = val;
 			valp++;
+			if (vumeter == VUMETER_STEREO)
+				c = !c;
 		}
 		break;
 	}
 	default:
 		if (run == 0) {
-			fprintf(stderr, _("Unsupported bit size %d.\n"), bits_per_sample);
+			fprintf(stderr, _("Unsupported bit size %d.\n"), (int)bits_per_sample);
 			run = 1;
 		}
 		return;
@@ -1197,49 +1358,61 @@ static void compute_max_peak(u_char *data, size_t count)
 	if (max <= 0)
 		max = 0x7fffffff;
 
-	if (bits_per_sample > 16)
-		perc = max_peak / (max / 100);
-	else
-		perc = max_peak * 100 / max;
+	for (c = 0; c < ichans; c++) {
+		if (bits_per_sample > 16)
+			perc[c] = max_peak[c] / (max / 100);
+		else
+			perc[c] = max_peak[c] * 100 / max;
+	}
 
-	if(verbose<=2) {
-		static int maxperc=0;
+	if (interleaved && verbose <= 2) {
+		static int maxperc[2];
 		static time_t t=0;
 		const time_t tt=time(NULL);
 		if(tt>t) {
 			t=tt;
-			maxperc=0;
+			maxperc[0] = 0;
+			maxperc[1] = 0;
 		}
-		if(perc>maxperc)
-			maxperc=perc;
+		for (c = 0; c < ichans; c++)
+			if (perc[c] > maxperc[c])
+				maxperc[c] = perc[c];
 
 		putchar('\r');
-		for (val = 0; val <= perc / 2 && val < 50; val++)
-			putchar('#');
-		for (; val <= maxperc / 2 && val < 50; val++)
-			putchar(' ');
-		putchar('+');
-		for (++val; val <= 50; val++)
-			putchar(' ');
-
-		printf("| %02i%%", maxperc);
-		if (perc>99)
-			printf(_(" !clip  "));
-
+		print_vu_meter(perc, maxperc);
 		fflush(stdout);
 	}
 	else if(verbose==3) {
-		printf(_("Max peak (%li samples): 0x%08x "), (long)ocount, max_peak);
+		printf(_("Max peak (%li samples): 0x%08x "), (long)ocount, max_peak[0]);
 		for (val = 0; val < 20; val++)
-			if (val <= perc / 5)
+			if (val <= perc[0] / 5)
 				putchar('#');
 			else
 				putchar(' ');
-		printf(" %i%%\n", perc);
+		printf(" %i%%\n", perc[0]);
 		fflush(stdout);
 	}
 }
 
+static void do_test_position(void)
+{
+	static int counter = 0;
+	snd_pcm_sframes_t avail, delay;
+	int err;
+
+	err = snd_pcm_avail_delay(handle, &avail, &delay);
+	if (err < 0)
+		return;
+	if (avail > 4 * (snd_pcm_sframes_t)buffer_frames ||
+	    avail < -4 * (snd_pcm_sframes_t)buffer_frames ||
+	    delay > 4 * (snd_pcm_sframes_t)buffer_frames ||
+	    delay < -4 * (snd_pcm_sframes_t)buffer_frames) {
+	  fprintf(stderr, "Suspicious buffer position (%i total): avail = %li, delay = %li, buffer = %li\n", ++counter, (long)avail, (long)delay, (long)buffer_frames);
+	} else if (verbose) {
+	  fprintf(stderr, "Buffer position: %li/%li (%li)\n", (long)avail, (long)delay, (long)buffer_frames);
+	}
+}
+
 /*
  *  write function
  */
@@ -1249,13 +1422,16 @@ static ssize_t pcm_write(u_char *data, size_t count)
 	ssize_t r;
 	ssize_t result = 0;
 
-	if (sleep_min == 0 &&
-	    count < chunk_size) {
+	if (count < chunk_size) {
 		snd_pcm_format_set_silence(hwparams.format, data + count * bits_per_frame / 8, (chunk_size - count) * hwparams.channels);
 		count = chunk_size;
 	}
 	while (count > 0) {
+		if (test_position)
+			do_test_position();
 		r = writei_func(handle, data, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
 			snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
@@ -1267,7 +1443,7 @@ static ssize_t pcm_write(u_char *data, size_t count)
 			exit(EXIT_FAILURE);
 		}
 		if (r > 0) {
-			if (verbose > 1)
+			if (vumeter)
 				compute_max_peak(data, r * hwparams.channels);
 			result += r;
 			count -= r;
@@ -1282,8 +1458,7 @@ static ssize_t pcm_writev(u_char **data, unsigned int channels, size_t count)
 	ssize_t r;
 	size_t result = 0;
 
-	if (sleep_min == 0 &&
-	    count != chunk_size) {
+	if (count != chunk_size) {
 		unsigned int channel;
 		size_t offset = count;
 		size_t remaining = chunk_size - count;
@@ -1297,7 +1472,11 @@ static ssize_t pcm_writev(u_char **data, unsigned int channels, size_t count)
 		size_t offset = result;
 		for (channel = 0; channel < channels; channel++)
 			bufs[channel] = data[channel] + offset * bits_per_sample / 8;
+		if (test_position)
+			do_test_position();
 		r = writen_func(handle, bufs, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
 			snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
@@ -1309,7 +1488,7 @@ static ssize_t pcm_writev(u_char **data, unsigned int channels, size_t count)
 			exit(EXIT_FAILURE);
 		}
 		if (r > 0) {
-			if (verbose > 1) {
+			if (vumeter) {
 				for (channel = 0; channel < channels; channel++)
 					compute_max_peak(data[channel], r);
 			}
@@ -1330,13 +1509,16 @@ static ssize_t pcm_read(u_char *data, size_t rcount)
 	size_t result = 0;
 	size_t count = rcount;
 
-	if (sleep_min == 0 &&
-	    count != chunk_size) {
+	if (count != chunk_size) {
 		count = chunk_size;
 	}
 
 	while (count > 0) {
+		if (test_position)
+			do_test_position();
 		r = readi_func(handle, data, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
 			snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
@@ -1348,7 +1530,7 @@ static ssize_t pcm_read(u_char *data, size_t rcount)
 			exit(EXIT_FAILURE);
 		}
 		if (r > 0) {
-			if (verbose > 1)
+			if (vumeter)
 				compute_max_peak(data, r * hwparams.channels);
 			result += r;
 			count -= r;
@@ -1364,8 +1546,7 @@ static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
 	size_t result = 0;
 	size_t count = rcount;
 
-	if (sleep_min == 0 &&
-	    count != chunk_size) {
+	if (count != chunk_size) {
 		count = chunk_size;
 	}
 
@@ -1375,7 +1556,11 @@ static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
 		size_t offset = result;
 		for (channel = 0; channel < channels; channel++)
 			bufs[channel] = data[channel] + offset * bits_per_sample / 8;
+		if (test_position)
+			do_test_position();
 		r = readn_func(handle, bufs, count);
+		if (test_position)
+			do_test_position();
 		if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
 			snd_pcm_wait(handle, 1000);
 		} else if (r == -EPIPE) {
@@ -1387,7 +1572,7 @@ static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
 			exit(EXIT_FAILURE);
 		}
 		if (r > 0) {
-			if (verbose > 1) {
+			if (vumeter) {
 				for (channel = 0; channel < channels; channel++)
 					compute_max_peak(data[channel], r);
 			}
@@ -1452,17 +1637,15 @@ static void voc_pcm_flush(void)
 {
 	if (buffer_pos > 0) {
 		size_t b;
-		if (sleep_min == 0) {
-			if (snd_pcm_format_set_silence(hwparams.format, audiobuf + buffer_pos, chunk_bytes - buffer_pos * 8 / bits_per_sample) < 0)
-				fprintf(stderr, _("voc_pcm_flush - silence error"));
-			b = chunk_size;
-		} else {
-			b = buffer_pos * 8 / bits_per_frame;
-		}
+		if (snd_pcm_format_set_silence(hwparams.format, audiobuf + buffer_pos, chunk_bytes - buffer_pos * 8 / bits_per_sample) < 0)
+			fprintf(stderr, _("voc_pcm_flush - silence error"));
+		b = chunk_size;
 		if (pcm_write(audiobuf, b) != (ssize_t)b)
 			error(_("voc_pcm_flush error"));
 	}
+	snd_pcm_nonblock(handle, 0);
 	snd_pcm_drain(handle);
+	snd_pcm_nonblock(handle, nonblock);
 }
 
 static void voc_play(int fd, int ofs, char *name)
@@ -1780,6 +1963,7 @@ static void begin_wave(int fd, size_t cnt)
 		bits = 16;
 		break;
 	case SND_PCM_FORMAT_S32_LE:
+        case SND_PCM_FORMAT_FLOAT_LE:
 		bits = 32;
 		break;
 	case SND_PCM_FORMAT_S24_LE:
@@ -1798,8 +1982,11 @@ static void begin_wave(int fd, size_t cnt)
 	cf.type = WAV_FMT;
 	cf.length = LE_INT(16);
 
-	f.format = LE_SHORT(WAV_PCM_CODE);
-	f.modus = LE_SHORT(hwparams.channels);
+        if (hwparams.format == SND_PCM_FORMAT_FLOAT_LE)
+                f.format = LE_SHORT(WAV_FMT_IEEE_FLOAT);
+        else
+                f.format = LE_SHORT(WAV_FMT_PCM);
+	f.channels = LE_SHORT(hwparams.channels);
 	f.sample_fq = LE_INT(hwparams.rate);
 #if 0
 	tmp2 = (samplesize == 8) ? 1 : 2;
@@ -1979,7 +2166,7 @@ void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *name)
 			if (r == 0)
 				break;
 			l += r;
-		} while (sleep_min == 0 && (size_t)l < chunk_bytes);
+		} while ((size_t)l < chunk_bytes);
 		l = l * 8 / bits_per_frame;
 		r = pcm_write(audiobuf, l);
 		if (r != l)
@@ -1988,7 +2175,9 @@ void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *name)
 		written += r;
 		l = 0;
 	}
+	snd_pcm_nonblock(handle, 0);
 	snd_pcm_drain(handle);
+	snd_pcm_nonblock(handle, nonblock);
 }
 
 
@@ -2219,7 +2408,7 @@ void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count,
 			if (r == 0)
 				break;
 			c += r;
-		} while (sleep_min == 0 && c < expected);
+		} while (c < expected);
 		c = c * 8 / bits_per_sample;
 		r = pcm_writev(bufs, channels, c);
 		if ((size_t)r != c)
@@ -2227,7 +2416,9 @@ void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count,
 		r = r * bits_per_frame / 8;
 		count -= r;
 	}
+	snd_pcm_nonblock(handle, 0);
 	snd_pcm_drain(handle);
+	snd_pcm_nonblock(handle, nonblock);
 }
 
 void capturev_go(int* fds, unsigned int channels, off64_t count, int rtype, char **names)
